import logging
from collections import defaultdict

logger = logging.getLogger(__name__)


class EventManager:
    __subscribers: dict = defaultdict(set)

    @classmethod
    def subscribe(cls, subscriber, event):
        cls.__subscribers[event].add(subscriber)

    @classmethod
    def trigger(cls, event):
        for subscriber in cls.__subscribers.get(event.__class__, []):
            subscriber(event)

    @classmethod
    def _reset(cls):
        """Never call this outside tests!"""
        cls.__subscribers = defaultdict(set)

    @classmethod
    def subscribers(cls):
        return dict(cls.__subscribers)



import json
import logging
from typing import Union

import boto3

from razemax.exceptions import DriverError


class Message:
    def __init__(self, id, event_name, receipt_handle, body):
        self.id = id
        self.event_name = event_name
        self.body = body
        self.receipt_handle = receipt_handle


class SQSDriver:
    def __init__(self, sqs_queue):
        self._queue = sqs_queue

    def receive_message(self) -> Union[Message, None]:
        message = self._queue.receive_messages(MaxNumberOfMessages=1)
        logging.info(f"Message received: {message}")

        if not message:
            return None

        message = message[0]
        message = self._process_message(message)

        return message

    def mark_message_processed(self, message: Message) -> None:
        self._queue.delete_messages(Entries=[{'Id': 'DummyId', 'ReceiptHandle': message.receipt_handle}])
        logging.info("Message {} deleted".format(message.id))

    def mark_message_unprocessed(self, message_id: Message, exception: Exception):
        # TODO: Add dead letter queue implementation
        pass

    @classmethod
    def _process_message(cls, message_sqs) -> Message:
        try:
            message_dict = json.loads(message_sqs.body)
            logging.info(f"Message body: {message_sqs.body}")

            message_content = json.loads(message_dict["Message"])
            message_id = message_dict["MessageId"]
            event_name = message_dict["MessageAttributes"]["event_name"]["Value"]
        except (KeyError,) as e:
            raise DriverError(e)
        except json.JSONDecodeError:
            raise DriverError("Message body not JSON encoded.")

        return Message(id=message_id,
                       event_name=event_name,
                       receipt_handle=message_sqs.receipt_handle,
                       body=message_content)

    @classmethod
    def build(cls, queue_name: str, aws_settings: dict):
        sqs_resource = boto3.resource('sqs', **aws_settings)
        sqs_queue = sqs_resource.get_queue_by_name(QueueName=queue_name)

        return cls(sqs_queue)



import json
import logging
from datetime import datetime

import boto3


class SNSMessagePublisher(object):
    def __init__(self, sns_client, topic_arn):
        self._sns_client = sns_client
        self._topic_arn = topic_arn

    def publish(self, event_name: str, event_body: dict, extra_meta: dict = {}):
        meta = {
            "timestamp": datetime.utcnow().isoformat('T'),
            "version": 1
        }
        meta.update(extra_meta)
        message = {
            "type": event_name,
            "meta": meta,
            "body": event_body
        }

        message_json = json.dumps(message)
        logging.info(f"event name {event_name}")

        return self._sns_client.publish(TopicArn=self._topic_arn,
                                        Message=message_json,
                                        MessageAttributes={
                                            'event_name': {
                                                'DataType': 'String',
                                                'StringValue': event_name
                                            }
                                        })

    @classmethod
    def build(cls, aws_settings: dict, topic_arn: str):
        """ aws_settings is a dict with:
            - region_name
            - aws_access_key_id
            - aws_secret_access_key
            - endpoint_url (optional)
        """
        sns_client = boto3.client('sns', **aws_settings)
        return cls(sns_client, topic_arn)



#!/usr/bin/env python


import io
import os

from setuptools import setup

NAME = 'razemax'
DESCRIPTION = 'SNS + SQS Communications made easy'
URL = f'https://github.com/21buttons/{NAME}'
EMAIL = 'jairo@21buttons.com'
AUTHOR = '21 Buttons APP S.L.'
REQUIRES_PYTHON = '>=3.6.0'
VERSION = None

REQUIRED = [
    'boto3'
]

EXTRAS = {}

here = os.path.abspath(os.path.dirname(__file__))

try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

about = {}
if not VERSION:
    with open(os.path.join(here, NAME, '__version__.py')) as f:
        exec(f.read(), about)
else:
    about['__version__'] = VERSION

setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=[NAME],
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    license="Apache License 2.0",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)



import logging
from typing import Union

from razemax.drivers import SQSDriver
from razemax.event_manager import EventManager


class MessageConsumer:
    def __init__(self, mapper_factory: dict, event_manager: Union[EventManager, type(EventManager)],
                 queue_driver: SQSDriver):
        self._mapper_factory = mapper_factory
        self._queue_driver = queue_driver
        self._event_manager = event_manager

    def process_message(self):
        # Receive message
        message = self._queue_driver.receive_message()

        if not message:
            logging.info("No messages to process")
            return None

        logging.info(f"Message type is: {message.event_name}")
        try:
            # Parse message to event
            mapper = self._mapper_factory[message.event_name]
            logging.info(f"Selected mapper is: {mapper}")

            event = mapper(message)
            logging.info(f"Event type is: {event.__class__}")

            # Trigger subscribers
            self._event_manager.trigger(event)
        except Exception as e:  # TODO: specify exceptions...
            logging.error(str(e))
            self._queue_driver.mark_message_unprocessed(message, e)
        else:
            self._queue_driver.mark_message_processed(message)
